@@ -3,20 +3,19 @@ package FFI::TinyCC;
 use strict;
 use warnings;
 use 5.008001;
+use Config;
 use FFI::Platypus;
 use FFI::Platypus::Memory qw( malloc free );
 use Carp qw( croak carp );
 use File::ShareDir ();
 
 # ABSTRACT: Tiny C Compiler for FFI
-our $VERSION = '0.17'; # VERSION
+our $VERSION = '0.18'; # VERSION
 
 
-sub _dlext
+sub _dlext ()
 {
-  require Config;
-  # recent strawberry Perl sets dlext to 'xs.dll'
-  $^O eq 'MSWin32' ? 'dll' : $Config::Config{dlext};
+  $^O eq 'MSWin32' ? 'dll' : $Config{dlext};
 }
 
 our $ffi = FFI::Platypus->new;
@@ -177,6 +176,50 @@ sub add_symbol
 }
 
 
+sub detect_sysinclude_path
+{
+  my($self) = @_;
+  if($^O eq 'MSWin32')
+  {
+    require File::Spec;
+    $self->add_sysinclude_path(File::Spec->catdir(File::ShareDir::dist_dir('Alien-TinyCC'), 'include'));
+  }
+  elsif($Config{incpth})
+  {
+    require Alien::TinyCC;
+    require File::Spec;
+    $self->add_sysinclude_path(File::Spec->catdir(Alien::TinyCC->libtcc_library_path, qw( tcc include )));
+    $self->add_sysinclude_path($_) for split /\s+/, $Config{incpth};
+  }
+  elsif($Config{ccname} eq 'gcc')
+  {
+    require File::Temp;
+    my($fh, $filename) = File::Temp::tempfile( "tryXXXX", SUFFIX => '.c', UNLINK => 1 );
+    close $fh;
+    
+    my @lines = `$Config{cpp} -v $filename 2>&1`;
+    
+    shift @lines while defined $lines[0] && $lines[0] !~ /^#include </;
+    shift @lines;
+    pop @lines while defined $lines[-1] && $lines[-1] !~ /^End of search /;
+    pop @lines;
+    
+    croak "Cannot detect sysinclude path" unless @lines;
+    
+    require Alien::TinyCC;
+    require File::Spec;
+    $self->add_sysinclude_path(File::Spec->catdir(Alien::TinyCC->libtcc_library_path, qw( tcc include )));
+    
+    $self->add_sysinclude_path($_) for map { chomp; s/^ //; $_ } @lines;
+  }    
+  else
+  {
+    croak "Cannot detect sysinclude path";
+  }
+}
+
+
+
 _method add_include_path => qw( string );
 
 
@@ -321,7 +364,7 @@ FFI::TinyCC - Tiny C Compiler for FFI
 
 =head1 VERSION
 
-version 0.17
+version 0.18
 
 =head1 SYNOPSIS
 
@@ -345,10 +388,10 @@ version 0.17
 
 =head1 DESCRIPTION
 
-This module provides an interface to a very small C compiler known as
-TinyCC.  It does almost no optimizations, so C<gcc> or C<clang> will
-probably generate faster code, but it is very small and is very fast
-and thus may be useful for some Just In Time (JIT) or Foreign Function
+This module provides an interface to a very small C compiler known as 
+TinyCC.  It does almost no optimizations, so C<gcc> or C<clang> will 
+probably generate faster code, but it is very small and is very fast and 
+thus may be useful for some Just In Time (JIT) or Foreign Function 
 Interface (FFI) situations.
 
 For a simpler, but less powerful interface see L<FFI::TinyCC::Inline>.
@@ -371,7 +414,8 @@ Methods will generally throw an exception on failure.
 
  $tcc->set_options($options);
 
-Set compiler and linker options, as you would on the command line, for example:
+Set compiler and linker options, as you would on the command line, for 
+example:
 
  $tcc->set_options('-I/foo/include -L/foo/lib -DFOO=22');
 
@@ -396,14 +440,14 @@ Compile a string containing C source code.
  $tcc->add_symbol($name, $callback);
  $tcc->add_symbol($name, $pointer);
 
-Add the given given symbol name / callback or pointer combination.
-See example below for how to use this to call Perl from Tiny C code.
+Add the given given symbol name / callback or pointer combination. See 
+example below for how to use this to call Perl from Tiny C code.
 
-It will accept a L<FFI::Raw::Callback> at a performance penalty.
-If possible pass in the pointer to the C entry point instead.
+It will accept a L<FFI::Raw::Callback> at a performance penalty. If 
+possible pass in the pointer to the C entry point instead.
 
-If you are using L<FFI::Platypus> you can use L<FFI::Platypus#cast>
-or L<FFI::Platypus::Declare#cast> to get a pointer to a closure:
+If you are using L<FFI::Platypus> you can use L<FFI::Platypus#cast> or 
+L<FFI::Platypus::Declare#cast> to get a pointer to a closure:
 
  use FFI::Platypus::Declare;
  my $clousre = closure { return $_[0]+1 };
@@ -413,6 +457,16 @@ or L<FFI::Platypus::Declare#cast> to get a pointer to a closure:
 
 =head2 Preprocessor options
 
+=head3 detect_sysinclude_path
+
+[version 0.18]
+
+ $tcc->detect_sysinclude_path;
+
+Attempt to find and configure the appropriate system include files. If 
+the platform that you are on does not (yet?) support this functionality 
+then this method will throw an exception.
+
 =head3 add_include_path
 
  $tcc->add_include_path($path);
@@ -423,7 +477,8 @@ Add the given path to the list of paths used to search for include files.
 
  $tcc->add_sysinclude_path($path);
 
-Add the given path to the list of paths used to search for system include files.
+Add the given path to the list of paths used to search for system 
+include files.
 
 =head3 set_lib_path
 
@@ -482,18 +537,18 @@ Add the given directory to the search path used to find libraries.
 
  my $pointer = $tcc->get_symbol($symbol_name);
 
-Return symbol address or undef if not found.  This can be passed into
-the L<FFI::Platypus#function> method, L<FFI::Platypus#attach> method,
-L<FFI::Platypus::Declare#function> function or similar interface that
+Return symbol address or undef if not found.  This can be passed into 
+the L<FFI::Platypus#function> method, L<FFI::Platypus#attach> method, 
+L<FFI::Platypus::Declare#function> function or similar interface that 
 takes a pointer to a C function.
 
 =head3 output_file
 
  $tcc->output_file($filename);
 
-Output the generated code (either executable, object or DLL) to the given filename.
-The type of output is specified by the L<set_output_type|FFI::TinyCC#set_output_type>
-method.
+Output the generated code (either executable, object or DLL) to the 
+given filename. The type of output is specified by the 
+L<set_output_type|/set_output_type> method.
 
 =head3 get_ffi_raw
 
@@ -501,20 +556,27 @@ B<DEPRECATED>
 
  my $ffi = $tcc->get_ffi_raw($symbol_name, $return_type, @argument_types);
 
-Given the name of a function, return an L<FFI::Raw> instance that will allow you to call it from Perl.
+Given the name of a function, return an L<FFI::Raw> instance that will 
+allow you to call it from Perl.
 
-This method is deprecated, and will be removed from a future version of
-L<FFI::TinyCC>.  It will issue a warning if you try to use it.  Instead
-of this:
+This method is deprecated, and will be removed from a future version of 
+L<FFI::TinyCC>, but not before January 31, 2017.  It will issue a 
+warning if you try to use it.  Instead of this:
 
- my $function = $ffi->get_ffi_raw($name, $ret, @args);
+ my $function = $ffi->get_ffi_raw($name, FFI::void);
+ $function->();
 
 Do this:
 
  use FFI::Raw;
- my $function = FFI::Raw->new_from_ptr($ffi->get_symbol($name), $ret, @args);
+ my $function = FFI::Raw->new_from_ptr($ffi->get_symbol($name), FFI::void);
+ $function->();
 
-Or better yet, use L<FFI::Platypus> instead.
+Or better yet, use L<FFI::Platypus> instead:
+
+ use FFI::Platypus::Declare;
+ attach [$ffi->get_symbol($name) => 'function'] => [] => 'void';
+ function();
 
 =head1 EXAMPLES
 
@@ -591,11 +653,12 @@ Or better yet, use L<FFI::Platypus> instead.
 
 =head1 CAVEATS
 
-Tiny C is only supported on platforms with ARM or Intel processors.  All features may not be fully supported on
-all operating systems.
+Tiny C is only supported on platforms with ARM or Intel processors.  All 
+features may not be fully supported on all operating systems.
 
-Tiny C is no longer supported by its original author, though various forks seem to have varying levels of support.
-We use the fork that comes with L<Alien::TinyCC>.
+Tiny C is no longer supported by its original author, though various 
+forks seem to have varying levels of support. We use the fork that comes 
+with L<Alien::TinyCC>.
 
 =head1 SEE ALSO
 
@@ -617,9 +680,9 @@ We use the fork that comes with L<Alien::TinyCC>.
 
 =head1 BUNDLED SOFTWARE
 
-This package also comes with a parser that was shamelessly stolen from L<XS::TCC>,
-which I strongly suspect was itself shamelessly "borrowed" from 
-L<Inline::C::Parser::RegExp>
+This package also comes with a parser that was shamelessly stolen from 
+L<XS::TCC>, which I strongly suspect was itself shamelessly "borrowed" 
+from L<Inline::C::Parser::RegExp>
 
 The license details for the parser are:
 
@@ -627,7 +690,8 @@ Copyright 2002 Brian Ingerson
 Copyright 2008, 2010-2012 Sisyphus
 Copyright 2013 Steffen Muellero
 
-This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
+This program is free software; you can redistribute it and/or modify it 
+under the same terms as Perl itself.
 
 =cut
 
